package ru.dbotthepony.mc.otm.menu.input

import net.minecraft.world.entity.player.Player
import ru.dbotthepony.mc.otm.client.minecraft
import ru.dbotthepony.mc.otm.menu.MatteryMenu
import kotlin.reflect.KMutableProperty0

abstract class AbstractNetworkedInput<V> {
	abstract val input: MatteryMenu.PlayerInput<V>
	abstract val value: V

	var isClient = false

	fun asClient(): AbstractNetworkedInput<V> {
		isClient = true
		supplier = null
		consumer = null
		return this
	}

	var supplier: (() -> V)? = null
	var consumer: ((V) -> Unit)? = null

	fun withSupplier(func: () -> V): AbstractNetworkedInput<V> {
		supplier = func
		return this
	}

	fun withConsumer(func: (V) -> Unit): AbstractNetworkedInput<V> {
		consumer = func
		return this
	}

	fun with(state: KMutableProperty0<V>): AbstractNetworkedInput<V> {
		withConsumer { state.set(it) }
		withSupplier { state.get() }
		return this
	}

	fun clear(): AbstractNetworkedInput<V> {
		supplier = null
		consumer = null
		return this
	}

	fun input(newValue: V, ply: Player? = null) {
		if (isClient) {
			input.checkedInput(newValue, ply)
		} else {
			consumer?.invoke(newValue)
		}
	}

	fun clientInput(newValue: V) {
		if (isClient) {
			input.checkedInput(newValue, minecraft.player)
		}
	}

	fun checkClient(): Boolean {
		return input.allowSpectators || minecraft.player?.isSpectator == false
	}
}
